import os
import random

import pyglet

import cocos
from cocos.director import director
from cocos.scenes.transitions import FadeTransition

from gamelib import game, ninepatch, euclid, curve

def main():
    global antialiasing
    # try to get a window with anti-aliasing but fall back to none if
    # necessary
    try:
        config = pyglet.gl.Config(sample_buffers=1, samples=4, 
                    depth_size=16, double_buffer=True,)
        cocos.director.director.init(width=900, height=600, config=config)
        antialiasing = True
    except pyglet.window.NoSuchConfigException:
        cocos.director.director.init(width=900, height=600)
        antialiasing = False

    w = cocos.director.director.window

    if __debug__:
        cocos.director.director.show_FPS = True

    pyglet.gl.glClearColor(255, 255, 255, 255)

    pyglet.resource.path.append(os.path.join(os.path.dirname(__file__),
        '..', 'data'))
    pyglet.resource.reindex()
    pyglet.resource.add_font('Ayuma2yk.ttf')

    cocos.director.director.run(build_menu())


class KeyPressLayer(cocos.layer.Layer):
    is_event_handler = True

    def on_key_press(self, sym, mods):
        if sym in (pyglet.window.key.ESCAPE, pyglet.window.key.SPACE):
            director.pop()
        else:
            return pyglet.event.EVENT_UNHANDLED
        return pyglet.event.EVENT_HANDLED


def make_help():
    layer = KeyPressLayer()
    w, h = director.get_window_size()
    im = cocos.sprite.Sprite('help-page.png')
    im.position = w//2, h//2
    layer.add(im)
    scroller = cocos.layer.ScrollingManager()
    background = game.Background()
    scroller.add(background, z=0)
    return cocos.scene.Scene(scroller, layer)


def build_menu():
    from cocos.menu import MenuItem, Menu
    menu = Menu("Endless Path")

    menu.menu_valign = cocos.menu.CENTER
    menu.menu_halign = cocos.menu.CENTER

    menu.font_title['color'] = (0x5d, 0x76, 0x1f, 255)
    menu.font_title['font_size'] = 128
    menu.font_title['font_name'] = 'A Yummy Apology'

    menu.font_item['color'] = (0x39, 0x13, 0x54, 255)
    menu.font_item['font_size'] = 64
    menu.font_item['font_name'] = 'A Yummy Apology'

    menu.font_item_selected['color'] = (0x63, 0x3a, 0x81, 255)
    menu.font_item_selected['font_size'] = 64
    menu.font_item_selected['font_name'] = 'A Yummy Apology'

    def play():
    #    pyglet.clock.unschedule(background.walker.update)
        director.push(FadeTransition(game.GameScene(),
            color=(0xc6, 0xda, 0x94)))

    def help():
        h = make_help()
    #    background.walker.scroller = h.children[0][1]
        director.push(h)

    menu.create_menu([
        MenuItem('Play', play),
        MenuItem('Help', help),
        MenuItem('Quit', pyglet.app.exit),
    ])
    menu.on_quit = pyglet.app.exit

    menubg = MenuBGLayer(menu)
        
    # some status info
    menubg.add(cocos.text.Label('euclid: %s'%['not accelerated',
        'accelerated'][euclid.accelerated], (5, 5), font_size=10, color=(0, 0, 0, 200)))
    menubg.add(cocos.text.Label('curves: %s'%['not accelerated',
        'accelerated'][curve.accelerated], (5, 17), font_size=10, color=(0, 0, 0, 200)))
    menubg.add(cocos.text.Label('anti-aliasing: %s'%['not activated',
        'activated'][antialiasing], (5, 29), font_size=10, color=(0, 0, 0, 200)))

    scroller = cocos.layer.ScrollingManager()
    background = game.Background()
    #background.walker = Walker(scroller)
    #pyglet.clock.schedule(background.walker.update)
    scroller.add(background, z=0)

    return cocos.scene.Scene(scroller, menubg, menu)

class Walker(object):
    def __init__(self, scroller):
        self.x, self.y = 0, 0
        self.scroller = scroller

        self.vector = euclid.Vector2(0, 0)
        self.accel_vector = euclid.Vector2(1, 0)
        self.accel = (random.random() - .5) * 100
        self.rotate = (random.random() - .5)
        self.rotate_accel = (random.random() - .5)/20

    def update(self, dt):
        v = self.vector
        r = euclid.Matrix3.new_rotate(self.rotate)
        v += r * self.accel_vector * (self.accel * dt)
        s = max(0, min(2, abs(v)))
        v.normalize()
        v *= s
        self.x, self.y = self.x + v.x, self.y + v.y
        self.scroller.set_focus(self.x, self.y)

        self.accel += (random.random() - .5) * 2
        self.rotate += self.rotate_accel
        self.rotate_accel += (random.random() - .5) /80


class MenuBGLayer(cocos.layer.Layer):
    def __init__(self, menu):
        super(MenuBGLayer, self).__init__()

        image = pyglet.resource.image('button-patch.png')
        self.patch = ninepatch.NinePatch(image)

        self.batch = pyglet.graphics.Batch()
        self.l = []
        for label in [menu.title_label] + [c[1].item for c in menu.children]:
            x = int(label.x - label.content_width/2)
            y = int(label.y - (label.content_height*.9)/2)
            self.l.append(self.patch.draw_around(self.batch, x, y,
                label.content_width,
                int(label.content_height-label.content_height*.25)))

    def draw(self):
        pyglet.gl.glPushMatrix()
        self.transform()
        self.batch.draw()
        pyglet.gl.glPopMatrix()


